@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -65,7 +65,7 @@ import com.openexchange.server.impl.OCLPermission;
 import com.openexchange.session.Session;
 import com.openexchange.unifiedinbox.UnifiedInboxAccess;
 import com.openexchange.unifiedinbox.UnifiedInboxException;
-import com.openexchange.unifiedinbox.services.UnifiedInboxServiceRegistry;
+import com.openexchange.unifiedinbox.services.Services;
 import com.openexchange.unifiedinbox.utility.UnifiedInboxUtility;
 
 /**
@@ -206,14 +206,15 @@ public final class UnifiedInboxFolderConverter {
     }
 
     public static void setPermissions(final MailFolder mailFolder, final int userId) {
-        final int fp = mailFolder.getOwnPermission().getFolderPermission();
+        MailPermission realPermission = mailFolder.getOwnPermission();
+        final int fp = realPermission.getFolderPermission();
         {
             final MailPermission ownPermission = new DefaultMailPermission();
             ownPermission.setEntity(userId);
             ownPermission.setGroupPermission(false);
             // Grant not more than OCLPermission.CREATE_OBJECTS_IN_FOLDER
             ownPermission.setFolderPermission(fp > OCLPermission.CREATE_OBJECTS_IN_FOLDER ? OCLPermission.CREATE_OBJECTS_IN_FOLDER : fp);
-            ownPermission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.DELETE_ALL_OBJECTS);
+            ownPermission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, realPermission.getWritePermission(), OCLPermission.DELETE_ALL_OBJECTS);
             ownPermission.setFolderAdmin(false);
             mailFolder.setOwnPermission(ownPermission);
         }
@@ -223,7 +224,7 @@ public final class UnifiedInboxFolderConverter {
             permission.setGroupPermission(true);
             // Grant not more than OCLPermission.CREATE_OBJECTS_IN_FOLDER
             permission.setFolderPermission(fp > OCLPermission.CREATE_OBJECTS_IN_FOLDER ? OCLPermission.CREATE_OBJECTS_IN_FOLDER : fp);
-            permission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.DELETE_ALL_OBJECTS);
+            permission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, realPermission.getWritePermission(), OCLPermission.DELETE_ALL_OBJECTS);
             permission.setFolderAdmin(false);
             mailFolder.removePermissions();
             mailFolder.addPermission(permission);
@@ -233,8 +234,7 @@ public final class UnifiedInboxFolderConverter {
     private static boolean setMessageCounts(final int unifiedInboxAccountId, final Session session, final MailFolder tmp) throws UnifiedInboxException, OXException {
         boolean retval = false;
         {
-            final MailAccountStorageService storageService =
-                UnifiedInboxServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
+            final MailAccountStorageService storageService = Services.getService(MailAccountStorageService.class);
             final MailAccount[] arr = storageService.getUserMailAccounts(session.getUserId(), session.getContextId());
             for (int i = 0; !retval && i < arr.length; i++) {
                 final MailAccount mailAccount = arr[i];
@@ -274,7 +274,7 @@ public final class UnifiedInboxFolderConverter {
             mailAccess = MailAccess.getInstance(session, accountId);
             mailAccess.connect();
             for (int i = 0; i < retval.length; i++) {
-                final String accountFullname = UnifiedInboxUtility.determineAccountFullname(mailAccess, fullnames[i]);
+                final String accountFullname = UnifiedInboxUtility.determineAccountFullName(mailAccess, fullnames[i]);
                 if (null != accountFullname && mailAccess.getFolderStorage().exists(accountFullname)) {
                     final MailFolder mf = mailAccess.getFolderStorage().getFolder(accountFullname);
                     retval[i] =
